import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { useCallback, useContext, useEffect, useState } from 'react';
import { ErrorContext } from '../hooks/error';
import { useRequest, useSSE } from '../tools';
import { DefaultSpinner, DefaultNotFound } from '../Defaults';
import { ConfigContext } from '../hooks/config';
import { usePageEventListen } from '../events';
import { EventContextProvider, useEventContext } from '../hooks/eventContext';
import { AnyCompList } from './index';
export const ServerLoadComp = ({ path, components, loadTrigger, sse }) => {
    if (components) {
        return _jsx(ServerLoadDefer, { path: path, components: components, loadTrigger: loadTrigger, sse: sse });
    }
    else if (sse) {
        return _jsx(ServerLoadSSE, { path: path });
    }
    else {
        return _jsx(ServerLoadFetch, { path: path });
    }
};
const ServerLoadDefer = ({ components, path, loadTrigger, sse, }) => {
    const { eventContext } = usePageEventListen(loadTrigger);
    if (eventContext) {
        return (_jsx(EventContextProvider, { context: eventContext, children: sse ? _jsx(ServerLoadSSE, { path: path }) : _jsx(ServerLoadFetch, { path: path }) }));
    }
    else {
        return _jsx(AnyCompList, { propsList: components });
    }
};
export const ServerLoadFetch = ({ path, devReload }) => {
    const [componentProps, setComponentProps] = useState(null);
    const [notFoundUrl, setNotFoundUrl] = useState(undefined);
    const url = useServerUrl(path);
    const request = useRequest();
    useEffect(() => {
        const promise = request({ url, expectedStatus: [200, 404] });
        promise.then(([status, data]) => {
            if (status === 200) {
                setComponentProps(data);
            }
            else {
                setNotFoundUrl(url);
            }
        });
        return () => {
            promise.then(() => null);
        };
    }, [url, request, devReload]);
    return _jsx(Render, { propsList: componentProps, notFoundUrl: notFoundUrl });
};
export const ServerLoadSSE = ({ path }) => {
    const [componentProps, setComponentProps] = useState(null);
    const url = useServerUrl(path);
    const onMessage = useCallback((data) => setComponentProps(data), []);
    useSSE(url, onMessage);
    return _jsx(Render, { propsList: componentProps });
};
const Render = ({ propsList, notFoundUrl }) => {
    const { error } = useContext(ErrorContext);
    const { Spinner, NotFound } = useContext(ConfigContext);
    const SpinnerComp = Spinner ?? DefaultSpinner;
    const NotFoundComp = NotFound ?? DefaultNotFound;
    if (notFoundUrl) {
        return _jsx(NotFoundComp, { url: notFoundUrl });
    }
    else if (propsList === null) {
        if (error) {
            return _jsx(_Fragment, {});
        }
        else {
            return _jsx(SpinnerComp, {});
        }
    }
    else {
        return _jsx(AnyCompList, { propsList: propsList });
    }
};
function useServerUrl(path) {
    const { rootUrl, pathSendMode } = useContext(ConfigContext);
    const applyContext = useEventContext();
    const requestPath = applyContext(path);
    if (pathSendMode === 'query') {
        return `${rootUrl}?path=${encodeURIComponent(requestPath)}`;
    }
    else {
        return rootUrl + requestPath;
    }
}
